@@ -24,6 +24,12 @@
 #include "desktop.h"
 #include "phosh.h"
 #include "render.h"
+#include "utils.h"
+
+/* help older (0.8.2) libxkbcommon */
+#ifndef XKB_KEY_XF86RotationLockToggle
+# define XKB_KEY_XF86RotationLockToggle 0x1008FFB7
+#endif
 
 struct phosh_private_keyboard_event_data {
   GHashTable *subscribed_accelerators;
@@ -42,38 +48,8 @@ phosh_rotate_display (struct wl_client   *client,
                       struct wl_resource *surface_resource,
                       uint32_t            degrees)
 {
-  struct phosh_private *phosh = wl_resource_get_user_data (resource);
-  enum wl_output_transform transform = WL_OUTPUT_TRANSFORM_NORMAL;
-
-  g_debug ("rotation: %d", degrees);
-  degrees %= 360;
-
-  switch (degrees) {
-  case 0:
-    transform =  WL_OUTPUT_TRANSFORM_NORMAL;
-    break;
-  case 90:
-    transform = WL_OUTPUT_TRANSFORM_90;
-    break;
-  case 180:
-    transform = WL_OUTPUT_TRANSFORM_180;
-    break;
-  case 270:
-    transform = WL_OUTPUT_TRANSFORM_270;
-    break;
-  default:
-    wl_resource_post_error (resource,
-                            PHOSH_PRIVATE_ERROR_INVALID_ARGUMENT,
-                            "Can only rotate in 90 degree steps");
-  }
-
-  if (!phosh->panel) {
-    g_warning ("Tried to rotate inexistent panel");
-    return;
-  }
-
-  wlr_output_set_transform (phosh->panel->output, transform);
-  output_damage_whole(phosh->panel->output->data);
+  wl_resource_post_error (resource, WL_DISPLAY_ERROR_INVALID_OBJECT,
+                          "Use wlr-output-management protocol instead");
 }
 
 
@@ -178,28 +154,17 @@ phosh_private_accelerator_already_subscribed (PhocKeyCombo *combo)
 }
 
 static bool
-keysym_is_media (xkb_keysym_t keysym)
+keysym_is_subscribeable (PhocKeyCombo *combo)
 {
-  switch (keysym) {
-  case XKB_KEY_XF86AudioLowerVolume:
-  case XKB_KEY_XF86AudioRaiseVolume:
-  case XKB_KEY_XF86AudioMute:
-  case XKB_KEY_XF86AudioMicMute:
-  case XKB_KEY_XF86AudioPlay:
-  case XKB_KEY_XF86AudioPause:
-  case XKB_KEY_XF86AudioStop:
-  case XKB_KEY_XF86AudioNext:
-  case XKB_KEY_XF86AudioPrev:
+  /* Allow to bind all keys with modifiers that aren't just shift/caps */
+  if (combo->modifiers >= WLR_MODIFIER_CTRL)
     return true;
-  default:
-    return false;
-  }
-}
 
-static bool
-keysym_is_subscribeable (xkb_keysym_t keysym)
-{
-  return keysym_is_media (keysym);
+  /* keys on multi media keyboards */
+  if (combo->keysym >= XKB_KEY_XF86MonBrightnessUp && combo->keysym <= XKB_KEY_XF86RotationLockToggle)
+    return true;
+
+  return false;
 }
 
 static void
@@ -225,7 +190,7 @@ phosh_private_keyboard_event_grab_accelerator_request (struct wl_client   *wl_cl
     return;
   }
 
-  if (!keysym_is_subscribeable (combo->keysym)) {
+  if (!keysym_is_subscribeable (combo)) {
     g_debug ("Requested keysym %s is not subscribeable!", accelerator);
 
     phosh_private_keyboard_event_send_grab_failed_event (resource,
@@ -262,6 +227,39 @@ phosh_private_keyboard_event_grab_accelerator_request (struct wl_client   *wl_cl
 
 }
 
+
+static void
+phosh_private_keyboard_event_ungrab_accelerator_request (struct wl_client *client,
+							 struct wl_resource *resource,
+							 uint32_t action_id)
+{
+  GHashTableIter iter;
+  gpointer key, value, found = NULL;
+  struct phosh_private_keyboard_event_data *kbevent =
+    phosh_private_keyboard_event_from_resource (resource);
+
+  g_debug ("Ungrabbing accelerator %d", action_id);
+  g_hash_table_iter_init (&iter, kbevent->subscribed_accelerators);
+  while (g_hash_table_iter_next (&iter, &key, &value)) {
+    if (GPOINTER_TO_INT (value) == action_id) {
+      found = key;
+      break;
+    }
+  }
+
+  if (found) {
+    g_hash_table_remove (kbevent->subscribed_accelerators, key);
+    phosh_private_keyboard_event_send_ungrab_success_event (resource,
+							    action_id);
+
+  } else {
+    phosh_private_keyboard_event_send_ungrab_failed_event (resource,
+							   action_id,
+							   PHOSH_PRIVATE_KEYBOARD_EVENT_ERROR_INVALID_ARGUMENT);
+  }
+}
+
+
 static void
 phosh_private_keyboard_event_handle_destroy (struct wl_client   *client,
                                              struct wl_resource *resource)
@@ -271,6 +269,7 @@ phosh_private_keyboard_event_handle_destroy (struct wl_client   *client,
 
 static const struct phosh_private_keyboard_event_interface phosh_private_keyboard_event_impl = {
   .grab_accelerator_request = phosh_private_keyboard_event_grab_accelerator_request,
+  .ungrab_accelerator_request = phosh_private_keyboard_event_ungrab_accelerator_request,
   .destroy = phosh_private_keyboard_event_handle_destroy
 };
 
@@ -303,7 +302,7 @@ handle_get_keyboard_event (struct wl_client   *client,
       g_free (kbevent);
       wl_client_post_no_memory (client);
       return;
-    }
+  }
 
   struct phosh_private *phosh_private = phosh_private_from_resource (phosh_private_resource);
 
